"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createCleanupsProvider = void 0;
const maps_1 = require("../../shared/maps");
const createFileNamesAndServices_1 = require("../createFileNamesAndServices");
const createSingleUseProvider_1 = require("../createSingleUseProvider");
const suppressTypeErrors_1 = require("../../cleanups/builtin/suppressTypeErrors");
const FileInfoCache_1 = require("../../shared/FileInfoCache");
const collectFilteredNodes_1 = require("../collectFilteredNodes");
const NameGenerator_1 = require("../../shared/NameGenerator");
const findMutationsInFile_1 = require("../findMutationsInFile");
/**
 * Creates a mutations provider that applies post-fix cleanups.
 *
 * @param allModifiedFileNames   Set to mark names of all files that were modified.
 * @returns Provider to apply post-fix cleanups, if needed.
 */
const createCleanupsProvider = (allModifiedFiles) => {
    return createSingleUseProvider_1.createSingleUseProvider("Applying post-fix cleanups", (options) => {
        if (!Object.values(options.cleanups).filter(Boolean).length) {
            return undefined;
        }
        return () => {
            const fileMutations = new Map();
            const { fileNames, services } = createFileNamesAndServices_1.createFileNamesAndServices(options);
            for (const fileName of fileNames) {
                const sourceFile = services.program.getSourceFile(fileName);
                if (sourceFile === undefined) {
                    options.output.stderr(`Could not find TypeScript source file for '${fileName}'.`);
                    continue;
                }
                const filteredNodes = collectFilteredNodes_1.collectFilteredNodes(options, sourceFile);
                const foundMutations = findMutationsInFile_1.findMutationsInFile({
                    fileInfoCache: new FileInfoCache_1.FileInfoCache(filteredNodes, services, sourceFile),
                    filteredNodes,
                    nameGenerator: new NameGenerator_1.NameGenerator(sourceFile.fileName),
                    options,
                    services,
                    sourceFile,
                }, [["suppressTypeIssues", suppressTypeErrors_1.suppressRemainingTypeIssues]]);
                if (foundMutations?.length) {
                    allModifiedFiles.add(fileName);
                    fileMutations.set(fileName, foundMutations);
                }
            }
            return {
                mutationsWave: {
                    fileMutations: fileMutations.size === 0 ? undefined : maps_1.convertMapToObject(fileMutations),
                },
            };
        };
    });
};
exports.createCleanupsProvider = createCleanupsProvider;
//# sourceMappingURL=createCleanupsProvider.js.map